<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';
require_once 'lib/tv/epg_iterator.php';
require_once 'lib/tv/default_channel.php';

class IzhcomTv extends AbstractTv {
    private $session;

    const REFRESH_TIMEOUT_SEC = 3600;

    public function __construct($session) {
        $this->session = $session;
        parent::__construct(
            AbstractTv::MODE_CHANNELS_1_TO_N,
            false,
            false);
    }

    private function too_old(&$plugin_cookies) {
    	if( !isset($plugin_cookies->tv_info_timestamp) 
    			|| $plugin_cookies->tv_info_timestamp < time() - IzhcomTv::REFRESH_TIMEOUT_SEC )
    		return(true);
		return(false);
    }
    
    public function ensure_channels_loaded(&$plugin_cookies) {
       	if (!isset($this->channels) || $this->too_old($plugin_cookies) ) {
			$this->load_channels($plugin_cookies);
			$plugin_cookies->tv_info_timestamp = time();
		}
	}

	protected function add_hd_channel(&$hd_group, &$channel) {
		if( !$hd_group )
			$hd_group = new DefaultGroup( '__hd_special_group', 'HD', $this->session->get_icon('hd_cat.png'));
		$hd_group->add_channel($channel);
		$channel->add_group($hd_group);
		return($hd_group);
	}

    protected function load_channels(&$plugin_cookies, $recurse = false) {
    	$this->session->check_logged_in($plugin_cookies, $recurse);

        $content = $this->session->http_get_document(C::IZHCOM_TV_INFO);
        
        $xml = simplexml_load_string($content);

        if ($xml === false) {
        	hd_print(__FUNCTION__ . ": error: can't parse XML document.");
			hd_print("XML-text: $doc.");
			throw new Exception('Illegal XML document');
		}
		if($xml->getName() != 'tv_info') {
			hd_print(__FUNCTION__ . ": error: unexpected node '" . $xml->getName() . "'. Expected: 'tv_info'");
			throw new Exception('Invalid XML document');
		}
		$exitcode = isset($xml->exitcode) ? (int)$xml->exitcode : -999;
        if( $exitcode != 0 ) {
        	if($recurse == false &&  $exitcode == -2 ) {
       			// Server side session possible destroyed. Try reactivate once.
       			return( $this->load_channels($plugin_cookies, true) );
			}
        	hd_print(__FUNCTION__ . ': tv info error (' . $xml->exitcode . '), ' . $xml->message);
        	throw new Exception('Server tv_info error');
		}
		
    	$this->channels = new HashedArray();
    	$this->groups = new HashedArray();

    	$hd_group = null;

		// Main group - first
		$main_group = null;
		for($i = 0; $i < count($xml->category); ++$i) {
			if($xml->category[$i]->id == 0) {
	            $main_group = $group = new DefaultGroup(
		            strval($xml->category[$i]->id),
		            strval($xml->category[$i]->name),
		            $this->session->get_icon(strval($xml->category[$i]->icon))
				);
                $this->groups->put($group);
                
                $channels = $xml->category[$i]->channels;
				for($j = 0; $j < count($channels); ++$j) {
					$id = strval($channels[$j]->id);
					$name = strval($channels[$j]->name);
					$icon = strval($channels[$j]->icon);
					$url = strval($channels[$j]->url);

					$ch = new DefaultChannel($id, $name, $this->session->get_icon($icon), $url);
					$this->channels->put($ch);
					
					$ch->add_group($group);
					$group->add_channel($ch);
				}
                break;
			}
		}
		
		// Other groups
		for($i = 0; $i < count($xml->category); ++$i) {
			if( $xml->category[$i]->id != 0 ) {
	    		$group = new DefaultGroup(
    		    		strval($xml->category[$i]->id),
    		    		strval($xml->category[$i]->name),
    		    		$this->session->get_icon(strval($xml->category[$i]->icon))
				);
				$this->groups->put($group);
				
				$channels = $xml->category[$i]->channels;
				for($j = 0; $j < count($channels); ++$j) {
					$id = strval($channels[$j]->id);
					$name = strval($channels[$j]->name);
					$icon = strval($channels[$j]->icon);
					$url = strval($channels[$j]->url);

					$ch = new DefaultChannel($id, $name, $this->session->get_icon($icon), $url);
					$this->channels->put($ch);

					$ch->add_group($group);
					$group->add_channel($ch);
					
					if($main_group) $main_group->add_channel($ch);
				}
			}
		}
		
		// HD
		for($i = 0; $i < count($xml->category); ++$i) {
			$channels = $xml->category[$i]->channels;
			for($j = 0; $j < count($channels); ++$j) {
				$name = strval($channels[$j]->name);
				if(strpos($name, 'HD') !== false) {
					$id = strval($channels[$j]->id);
					$icon = strval($channels[$j]->icon);
					$url = strval($channels[$j]->url);
					
					$ch_hd = new DefaultChannel($id . '_hd', $name, $this->session->get_icon($icon), $url);
					$this->channels->put($ch_hd);
					$this->add_hd_channel($hd_group, $ch_hd);
				}
			}
		}
		
		if($hd_group) $this->groups->put($hd_group);
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies) {
    	$this->session->check_logged_in($plugin_cookies);
        return parent::get_tv_info($media_url, $plugin_cookies);
    }

    protected function load_epg($channel_id, $day_start_ts, &$plugin_cookies, $recurse = false) {
		$this->session->check_logged_in($plugin_cookies, $recurse);
		$this->ensure_channels_loaded($plugin_cookies);
		
		$content = $this->session->http_get_document( sprintf(C::IZHCOM_EPG_INFO, $channel_id, $day_start_ts) );
		$xml = simplexml_load_string($content);
        
        if ($xml === false) {
        	hd_print(__FUNCTION__ . ": error: can't parse XML document.");
			hd_print("XML-text: $doc.");
			throw new Exception('Illegal XML document');
		}
		if($xml->getName() != 'epg_info') {
			hd_print(__FUNCTION__ . ": error: unexpected node '" . $xml->getName() . "'. Expected: 'epg_info'");
			throw new Exception('Invalid XML document');
		}
		$exitcode = isset($xml->exitcode) ? (int)$xml->exitcode : -999;
        if( $exitcode != 0 ) {
        	if($recurse == false &&  $exitcode == -2 ) {
       			// Server side session possible destroyed. Try reactivate once.
       			return( $this->load_epg($channel_id, $day_start_ts, $plugin_cookies, true) );
			}
        	hd_print(__FUNCTION__ . ': epg_info error (' . $xml->exitcode . '), ' . $xml->message);
        	throw new Exception('Server epg_info error');
		} 

		$epg_items = array();

        foreach ($xml->epg as $epg) {
            $name = strval($epg->progname);
            $description = '';
            $start = intval($epg->ut_start);
            $end = -1;
            $epg_items[] = new DefaultEpgItem( $name, $description, $start, $end );
        }

        return new EpgIterator($epg_items, $day_start_ts, $day_start_ts + 86400);
	}				

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies) {
    	return( $this->load_epg($channel_id, $day_start_ts, $plugin_cookies) );
	}     
	
	public function get_fav_icon_url() {
		return ('');
	}

	public function get_archive(MediaURL $media_url) {
		return( $this->session->get_archive() );
	}

	public function get_tv_stream_url($playback_url, &$plugin_cookies) {
		hd_print(__FUNCTION__ . ' : ' . $playback_url);
		return $this->session->api_get_stream_url($playback_url);
	}
	
	public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies) {
		return( parent::get_tv_playback_url($channel_id, $archive_ts, $protect_code, $plugin_cookies) );
	}
	
	public function add_special_groups(&$items) {
        array_unshift($items,
            array
            (
                PluginRegularFolderItem::media_url =>
                    MediaURL::encode(
                        array
                        (
                            'screen_id' => IzhcomVodRootScreen::ID,
                        )),
                PluginRegularFolderItem::caption => TEXT_CATEGORY_VOD,
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path =>
                        $this->session->get_icon('vod_cat.png')
                )
            ));
    }
}
?>
